@@ -1,4 +1,5 @@
 import pytest
+import time
 import datetime
 import loguru
 from pytest_mock import MockerFixture
@@ -6,7 +7,7 @@ import shioaji as sj
 
 from decimal import Decimal
 from dataclasses import dataclass
-from sjtrade.trader import Position, SJTrader
+from sjtrade.trader import Position, PositionCond, SJTrader, SimulationShioaji
 from shioaji.constant import (
     Action,
     TFTStockPriceType,
@@ -16,6 +17,7 @@ from shioaji.constant import (
     OrderState,
     TFTStockOrderLot,
     StockOrderCond,
+    StockFirstSell,
 )
 
 
@@ -32,6 +34,11 @@ def sjtrader(api: sj.Shioaji) -> SJTrader:
     return SJTrader(api)
 
 
+@pytest.fixture
+def sjtrader_sim(api: sj.Shioaji) -> SJTrader:
+    return SJTrader(api, simulation=True)
+
+
 @pytest.fixture
 def positions():
     return {"1605": -1, "6290": -3, "0000": 3}
@@ -50,6 +57,12 @@ def sjtrader_entryed(sjtrader: SJTrader, positions: dict):
     return sjtrader
 
 
+@pytest.fixture
+def sjtrader_entryed_sim(sjtrader_sim: SJTrader, positions: dict) -> SJTrader:
+    res = sjtrader_sim.place_entry_order(positions, 1.05)
+    return sjtrader_sim
+
+
 def test_sjtrader(api: sj.Shioaji):
     sjtrader = SJTrader(api)
     assert hasattr(sjtrader, "api")
@@ -78,6 +91,11 @@ def test_sjtrader_start(sjtrader: SJTrader, mocker: MockerFixture, positions: di
     )
 
 
+def test_sjtrader_sj_event_handler(sjtrader: SJTrader, logger: loguru._logger.Logger):
+    sjtrader.sj_event_handel(0, 0, "info", "event")
+    logger.info.assert_called_once()
+
+
 def test_sjtrader_place_entry_order(
     sjtrader: SJTrader, logger: loguru._logger.Logger, positions: dict
 ):
@@ -94,6 +112,7 @@ def test_sjtrader_place_entry_order(
             ((sjtrader.api.Contracts.Stocks["6290"],), dict(version=QuoteVersion.v1)),
         ]
     )
+    assert logger.info.call_count == 2
     sjtrader.api.update_status.assert_called_once()
     expected = [
         sj.order.Trade(
@@ -105,6 +124,7 @@ def test_sjtrader_place_entry_order(
                 price_type=TFTStockPriceType.LMT,
                 order_type=TFTOrderType.ROD,
                 custom_field=sjtrader.name,
+                first_sell=StockFirstSell.Yes,
             ),
             status=sj.order.OrderStatus(status=sj.order.Status.PreSubmitted),
         ),
@@ -116,6 +136,7 @@ def test_sjtrader_place_entry_order(
                 action=Action.Sell,
                 price_type=TFTStockPriceType.LMT,
                 order_type=TFTOrderType.ROD,
+                first_sell=StockFirstSell.Yes,
                 custom_field=sjtrader.name,
             ),
             status=sj.order.OrderStatus(status=sj.order.Status.PreSubmitted),
@@ -124,9 +145,11 @@ def test_sjtrader_place_entry_order(
     assert sjtrader.positions == {
         "1605": Position(
             contract=sjtrader.api.Contracts.Stocks["1605"],
-            quantity=-1,
-            stop_loss_price=42.7,
-            stop_profit_price=35.9,
+            cond=PositionCond(
+                quantity=-1,
+                stop_loss_price=42.7,
+                stop_profit_price=35.9,
+            ),
             entry_trades=[
                 expected[0],
             ],
@@ -135,9 +158,11 @@ def test_sjtrader_place_entry_order(
         ),
         "6290": Position(
             contract=sjtrader.api.Contracts.Stocks["6290"],
-            quantity=-3,
-            stop_loss_price=62.1,
-            stop_profit_price=52.2,
+            cond=PositionCond(
+                quantity=-3,
+                stop_loss_price=62.1,
+                stop_profit_price=52.2,
+            ),
             entry_trades=[
                 expected[1],
             ],
@@ -150,7 +175,7 @@ def test_sjtrader_place_entry_order(
 
 
 def test_sjtrader_cancel_preorder_handler(
-    sjtrader_entryed: SJTrader, mocker: MockerFixture
+    sjtrader_entryed: SJTrader, mocker: MockerFixture, logger: loguru._logger.Logger
 ):
     tick = TickSTKv1("1605", "2022-05-25 08:45:01", 43.3, True)
 
@@ -166,6 +191,7 @@ def test_sjtrader_cancel_preorder_handler(
     sjtrader_entryed.api.cancel_order.assert_called_once_with(
         sjtrader_entryed.positions["1605"].entry_trades[0]
     )
+    assert logger.info.called
     # TODO need single trade update status interface
     # sjtrader_entryed.update_status.assert_called_once_with(
     #     sjtrader_entryed.positions["1605"].entry_trades[0]
@@ -175,8 +201,7 @@ def test_sjtrader_cancel_preorder_handler(
 
 
 def test_sjtrader_re_entry_order(
-    sjtrader_entryed: SJTrader,
-    mocker: MockerFixture,
+    sjtrader_entryed: SJTrader, mocker: MockerFixture, logger: loguru._logger.Logger
 ):
     tick = TickSTKv1("1605", "2022-05-25 08:45:01", 43.3, True)
 
@@ -200,27 +225,35 @@ def test_sjtrader_re_entry_order(
             action=Action.Sell,
             price_type=TFTStockPriceType.MKT,
             order_type=TFTOrderType.ROD,
+            first_sell=StockFirstSell.Yes,
             custom_field="dt1",
         ),
     )
+    assert logger.info.called
 
 
-def test_sjtrader_stop_profit(sjtrader_entryed: SJTrader, mocker: MockerFixture):
+def test_sjtrader_stop_profit(
+    sjtrader_entryed: SJTrader, mocker: MockerFixture, logger: loguru._logger.Logger
+):
     tick = TickSTKv1("1605", "2022-05-25 08:45:01", 35.5, False)
     position = sjtrader_entryed.positions["1605"]
-    position.open_quantity = -1
+    position.status.open_quantity = -1
     sjtrader_entryed.place_cover_order = mocker.MagicMock()
     sjtrader_entryed.stop_profit(position, tick)
     sjtrader_entryed.place_cover_order.assert_called_once()
+    assert logger.info.called
 
 
-def test_sjtrader_stop_loss(sjtrader_entryed: SJTrader, mocker: MockerFixture):
+def test_sjtrader_stop_loss(
+    sjtrader_entryed: SJTrader, mocker: MockerFixture, logger: loguru._logger.Logger
+):
     tick = TickSTKv1("1605", "2022-05-25 08:45:01", 43.3, False)
     position = sjtrader_entryed.positions["1605"]
-    position.open_quantity = -1
+    position.status.open_quantity = -1
     sjtrader_entryed.place_cover_order = mocker.MagicMock()
     sjtrader_entryed.stop_loss(position, tick)
     sjtrader_entryed.place_cover_order.assert_called_once()
+    assert logger.info.called
 
 
 def test_sjtrader_intraday_handler(sjtrader_entryed: SJTrader, mocker: MockerFixture):
@@ -234,13 +267,16 @@ def test_sjtrader_intraday_handler(sjtrader_entryed: SJTrader, mocker: MockerFix
     sjtrader_entryed.stop_loss.assert_called_once()
 
 
-def test_sjtrader_place_cover_order(sjtrader_entryed: SJTrader):
+def test_sjtrader_place_cover_order(
+    sjtrader_entryed: SJTrader, logger: loguru._logger.Logger
+):
     position = sjtrader_entryed.positions["1605"]
-    position.open_quantity = -1
+    position.status.open_quantity = -1
     sjtrader_entryed.place_cover_order(position)
     order_msg = gen_sample_order_msg("1605", Action.Buy, 1, op_type="New", op_code="00")
     sjtrader_entryed.order_handler(order_msg, position)
-    assert position.cover_order_quantity == 1
+    assert position.status.cover_order_quantity == 1
+    position.cover_trades[0].order.price == 43.3
     # TODO assert called once with
     # sjtrader_entryed.api.place_order.assert_has_calls([
     #     ((), dict(contract=position.contract, order=sj.order.TFTStockOrder(
@@ -249,15 +285,19 @@ def test_sjtrader_place_cover_order(sjtrader_entryed: SJTrader):
     #     )))
     # ])
     assert len(position.cover_trades) == 1
+    assert logger.info.called
 
 
-def test_sjtrader_open_position_cover(sjtrader_entryed: SJTrader):
+def test_sjtrader_open_position_cover(
+    sjtrader_entryed: SJTrader, logger: loguru._logger.Logger
+):
     position = sjtrader_entryed.positions["1605"]
     position.open_quantity = -1
     sjtrader_entryed.place_cover_order(position)
     order_msg = gen_sample_order_msg("1605", Action.Buy, 1, op_type="New", op_code="00")
     sjtrader_entryed.order_handler(order_msg, position)
     sjtrader_entryed.open_position_cover()
+    assert logger.info.called
 
 
 @pytest.fixture
@@ -391,47 +431,204 @@ def test_sjtrader_order_deal_handler_receive_deal_msg(
     )
 
 
-def test_sjtrader_order_handler(sjtrader_entryed: SJTrader):
+def test_sjtrader_order_handler(
+    sjtrader_entryed: SJTrader, logger: loguru._logger.Logger
+):
     position = sjtrader_entryed.positions["1605"]
     order_msg = gen_sample_order_msg(
         "1605", Action.Sell, 1, op_type="New", op_code="00"
     )
     sjtrader_entryed.order_handler(order_msg, position)
-    assert position.entry_order_quantity == -1
+    assert position.status.entry_order_quantity == -1
 
     order_msg = gen_sample_order_msg(
         "1605", Action.Sell, 1, op_type="Cancel", op_code="00"
     )
     sjtrader_entryed.order_handler(order_msg, position)
-    assert position.entry_order_quantity == 0
+    assert position.status.entry_order_quantity == 0
 
     order_msg = gen_sample_order_msg("1605", Action.Buy, 1, op_type="New", op_code="00")
     sjtrader_entryed.order_handler(order_msg, position)
-    assert position.cover_order_quantity == 1
+    assert position.status.cover_order_quantity == 1
 
     order_msg = gen_sample_order_msg(
         "1605", Action.Buy, 1, op_type="Cancel", op_code="00"
     )
     sjtrader_entryed.order_handler(order_msg, position)
-    assert position.cover_order_quantity == 0
+    assert position.status.cover_order_quantity == 0
+    assert logger.info.called
 
 
-def test_sjtrader_deal_handler(sjtrader_entryed: SJTrader):
+def test_sjtrader_deal_handler(
+    sjtrader_entryed: SJTrader, logger: loguru._logger.Logger
+):
     position = sjtrader_entryed.positions["1605"]
     deal_msg = gen_sample_deal_msg("1605", Action.Sell, 1)
     sjtrader_entryed.deal_handler(deal_msg, position)
-    assert position.open_quantity == -1
-    assert position.entry_quantity == -1
+    assert position.status.open_quantity == -1
+    assert position.status.entry_quantity == -1
     deal_msg = gen_sample_deal_msg("1605", Action.Buy, 1)
     sjtrader_entryed.deal_handler(deal_msg, position)
-    assert position.open_quantity == 0
-    assert position.cover_quantity == 1
+    assert position.status.open_quantity == 0
+    assert position.status.cover_quantity == 1
+    assert logger.info.called
 
 
-def test_sjtrader_update_status(sjtrader_entryed: SJTrader):
-
-    sjtrader_entryed.update_status(sjtrader_entryed.positions["1605"].entry_trades[0])
-
-    sjtrader_entryed.api._solace.update_status.assert_called_with(
-        sjtrader_entryed.positions["1605"].entry_trades[0].order.account, seqno=""
+def test_sim_sj_quote_callback(api: sj.Shioaji):
+    sim_api = SimulationShioaji(print)
+    contract = api.Contracts.Stocks["1605"]
+    order = sj.Order(
+        price=35,
+        quantity=1,
+        action=Action.Sell,
+        price_type=TFTStockPriceType.LMT,
+        order_type=TFTOrderType.ROD,
+        custom_field="dt1",
     )
+    sim_api.place_order(
+        contract,
+        order,
+    )
+    order = sj.Order(
+        price=34,
+        quantity=1,
+        action=Action.Sell,
+        price_type=TFTStockPriceType.LMT,
+        order_type=TFTOrderType.ROD,
+        custom_field="dt1",
+    )
+    sim_api.place_order(
+        contract,
+        order,
+    )
+    time.sleep(1.1)
+    tick = TickSTKv1("1605", "2022-05-25 09:00:01", 34.5, False)
+    assert len(sim_api.lmt_price_trades["1605"]) == 2
+    sim_api.quote_callback(Exchange.TSE, tick)
+    assert sim_api.snapshots["1605"].price == 34.5
+    assert len(sim_api.lmt_price_trades["1605"]) == 1
+    tick = TickSTKv1("1605", "2022-05-25 09:00:02", 36, False)
+    sim_api.quote_callback(Exchange.TSE, tick)
+    assert sim_api.snapshots["1605"].price == 36
+    assert "1605" not in sim_api.lmt_price_trades
+
+
+def test_sim_sj_place_order(api: sj.Shioaji):
+    sim_api = SimulationShioaji(print)
+    contract = api.Contracts.Stocks["1605"]
+    order = sj.Order(
+        price=41.35,
+        quantity=1,
+        action=Action.Sell,
+        price_type=TFTStockPriceType.MKT,
+        order_type=TFTOrderType.ROD,
+        custom_field="dt1",
+    )
+    trade = sim_api.place_order(
+        contract,
+        order,
+    )
+    time.sleep(0.55)
+    assert trade.contract == contract
+    # assert trade.order == order
+    assert trade.order.seqno == "000001"
+    assert trade.order.id == "c2c49415"
+    assert trade.order.ordno != ""
+    assert trade.status.status == "Submitted"
+    time.sleep(0.1)
+    assert trade.status.deal_quantity == 1
+
+
+def test_sim_sj_cancel_order(api: sj.Shioaji):
+    sim_api = SimulationShioaji(print)
+    contract = api.Contracts.Stocks["1605"]
+    order = sj.Order(
+        price=41.35,
+        quantity=1,
+        action=Action.Sell,
+        price_type=TFTStockPriceType.LMT,
+        order_type=TFTOrderType.ROD,
+        custom_field="dt1",
+    )
+    trade = sim_api.place_order(
+        contract,
+        order,
+    )
+    time.sleep(0.55)
+    trade = sim_api.cancel_order(trade)
+    time.sleep(0.55)
+    assert trade.status.status == sj.order.Status.Cancelled
+
+    order = sj.Order(
+        price=41.35,
+        quantity=1,
+        action=Action.Sell,
+        price_type=TFTStockPriceType.MKT,
+        order_type=TFTOrderType.ROD,
+        custom_field="dt1",
+    )
+    trade = sim_api.place_order(
+        contract,
+        order,
+    )
+    time.sleep(0.55)
+    trade = sim_api.cancel_order(trade)
+    time.sleep(0.55)
+    assert trade.status.status == sj.order.Status.Filled
+
+
+def test_sim_sj_update_status(api: sj.Shioaji):
+    sim_api = SimulationShioaji(print)
+    sim_api.update_status()
+
+
+def test_sjtrader_sim(api: sj.Shioaji):
+    sjtrader = SJTrader(api, simulation=True)
+    assert hasattr(sjtrader, "simulation_api")
+
+
+def test_sjtrader_sim_place_entry_order(
+    sjtrader_sim: SJTrader, logger: loguru._logger.Logger, positions: dict
+):
+    res = sjtrader_sim.place_entry_order(positions, 1.05)
+    logger.warning.assert_called_once()
+    sjtrader_sim.api.quote.subscribe.assert_has_calls(
+        [
+            (
+                (sjtrader_sim.api.Contracts.Stocks["1605"],),
+                dict(version=QuoteVersion.v1),
+            ),
+            (
+                (sjtrader_sim.api.Contracts.Stocks["6290"],),
+                dict(version=QuoteVersion.v1),
+            ),
+        ]
+    )
+    assert logger.info.call_count == 2
+    assert len(res) == 2
+
+
+def test_sjtrader_sim_cancel_preorder_handler(
+    sjtrader_entryed_sim: SJTrader, logger: loguru._logger.Logger
+):
+    tick = TickSTKv1("1605", "2022-05-25 08:45:01", 43.3, True)
+    sjtrader_entryed_sim.cancel_preorder_handler(Exchange.TSE, tick)
+
+    assert logger.info.called
+    time.sleep(0.55)
+    assert sjtrader_entryed_sim.positions["1605"].status.cancel_quantity == 1
+
+
+def test_sjtrader_sim_re_entry_order(
+    sjtrader_entryed_sim: SJTrader, logger: loguru._logger.Logger
+):
+    tick = TickSTKv1("1605", "2022-05-25 08:45:01", 43.3, True)
+    position = sjtrader_entryed_sim.positions["1605"]
+    sjtrader_entryed_sim.cancel_preorder_handler(position, tick)
+    time.sleep(0.55)
+    assert sjtrader_entryed_sim.positions["1605"].status.cancel_preorder == True
+    tick = TickSTKv1("1605", "2022-05-25 09:00:01", 35, False)
+    sjtrader_entryed_sim.re_entry_order(position, tick)
+    assert logger.info.called
+    time.sleep(0.65)
+    assert sjtrader_entryed_sim.positions["1605"].status.entry_order_quantity == -1
